#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// Add this to the declarations. It allows for the use of ArrayLists. (Used in the Level II book creation process.)
using System.Collections.Generic;

// Add this to the declarations. It allows the use of a Timer object to trigger events at our own time.
using System.Windows.Forms;

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Reference sample demonstrating how to use custom events
    /// </summary>
    [Description("Reference sample demonstrating how to use custom events")]
    public class SampleCustomEvents : Indicator
    {
        #region Variables
		private	List<LadderRow>		askRows	= new List<LadderRow>();
		private	List<LadderRow>		bidRows	= new List<LadderRow>();
		
		private bool firstAskEvent	= true;
		private bool firstBidEvent	= true;
				
		private System.Windows.Forms.Timer myTimer = new System.Windows.Forms.Timer();		
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Overlay				= true;
            PriceTypeSupported	= false;
        }

		// This collapsed section contains code to generate your own Level II book as shown in the SampleMarketDepth reference.
		#region Level II Book		
		/// <summary>
		/// This class holds a row of the bid or ask ladder.
		/// </summary>
		private class LadderRow
		{
			public	string	MarketMaker;			// relevant for stocks only
			public	double	Price;
			public	long	Volume;

			public LadderRow(double myPrice, long myVolume, string myMarketMaker)
			{
				MarketMaker	= myMarketMaker;
				Price		= myPrice;
				Volume		= myVolume;
			}
		}
		
		/// <summary>
        /// Called on each incoming real time market depth event
        /// </summary>
        protected override void OnMarketDepth(MarketDepthEventArgs e)
        {
			List<LadderRow> rows = null;

			// Checks to see if the Market Data is of the Ask type
			if (e.MarketDataType == MarketDataType.Ask)
			{
				rows = askRows;
				
				// Due to race conditions, it is possible the first event is an Update operation instead of an Insert. When this happens, populate your Lists via e.MarketDepth first.
				if (firstAskEvent)
				{
					if (e.Operation == Operation.Update)
					{
						// Lock the MarketDepth collection to prevent modification to the collection while we are still processing it
						lock (e.MarketDepth.Ask)
						{
							for (int idx = 0; idx < e.MarketDepth.Ask.Count; idx++)
								rows.Add(new LadderRow(e.MarketDepth.Ask[idx].Price, e.MarketDepth.Ask[idx].Volume, e.MarketDepth.Ask[idx].MarketMaker));
						}
					}
					firstAskEvent = false;
				}
			}
			
			// Checks to see if the Market Data is of the Bid type
			else if (e.MarketDataType == MarketDataType.Bid)
			{
				rows = bidRows;
				
				// Due to race conditions, it is possible the first event is an Update operation instead of an Insert. When this happens, populate your Lists via e.MarketDepth first.
				if (firstBidEvent)
				{
					if (e.Operation == Operation.Update)
					{
						// Lock the MarketDepth collection to prevent modification to the collection while we are still processing it
						lock (e.MarketDepth.Bid)
						{
							for (int idx = 0; idx < e.MarketDepth.Bid.Count; idx++)
								rows.Add(new LadderRow(e.MarketDepth.Bid[idx].Price, e.MarketDepth.Bid[idx].Volume, e.MarketDepth.Bid[idx].MarketMaker));
						}
					}
					firstBidEvent = false;
				}
			}

			if (rows == null)
				return;

			// Checks to see if the action taken was an insertion into the ladder
			if (e.Operation == Operation.Insert)
			{
				// Add a new row at the end if the designated position is greater than our current ladder size
				if (e.Position >= rows.Count)
					rows.Add(new LadderRow(e.Price, e.Volume, e.MarketMaker));
				
				// Insert a new row into our ladder at the designated position
				else
					rows.Insert(e.Position, new LadderRow(e.Price, e.Volume, e.MarketMaker));
			}
			
			/* Checks to see if the action taken was a removal of itself from the ladder
			Note: Due to the multi threaded architecture of the NT core, race conditions could occur
			-> check if e.Position is within valid range */
			else if (e.Operation == Operation.Remove && e.Position < rows.Count)
				rows.RemoveAt(e.Position);
			
			/* Checks to see if the action taken was to update a data already on the ladder
			Note: Due to the multi threaded architecture of the NT core, race conditions could occur
			-> check if e.Position is within valid range */
			else if (e.Operation == Operation.Update && e.Position < rows.Count)
			{
				rows[e.Position].MarketMaker	= e.MarketMaker;
				rows[e.Position].Price			= e.Price;
				rows[e.Position].Volume			= e.Volume;
			}
        }
		#endregion
		
        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnStartUp()
        {
			// Initiate our Timer object with an interval of 1000ms (1 second)
			myTimer.Tick += new EventHandler(TimerEventProcessor);
			myTimer.Interval = 1000;
			myTimer.Start();
		
        }
		
		// Called on each bar update event (incoming tick).
		protected override void OnBarUpdate()
		{
			// Your custom OnBarUpdate logic would be placed in here
		}
		
		
		// Timer's tick event handler. Called at every tick of 1000ms.
		private void TimerEventProcessor(Object myObject, EventArgs myEventArgs)
		{
			/* Important to use the TriggerCustomEvent() to ensure that NinjaScript indexes and pointers are correctly set.
			Do not process your code here. Process your code in the MyCustomHandler method. */
			TriggerCustomEvent(MyCustomHandler, 0, myTimer.Interval);
		}
		
		private void MyCustomHandler(object state)
		{
			// Informs us of when the event was triggered and of our Timer settings
			Print("\tTime: " + DateTime.Now);
			Print("\tTimer Interval: " + state.ToString() + "ms");
			
			// Prints the L2 Ask Book we created. Cycles through the whole List and prints the contained objects.
			Print("Ask Book: ");
			for (int idx = 0; idx < askRows.Count; idx++)
				Print("Ask Price=" + askRows[idx].Price + " Volume=" + askRows[idx].Volume + " Position=" + idx);
			
			// Prints the L2 Bid Book we created. Cycles through the whole List and prints the contained objects.
			Print("Bid Book: ");
			for (int idx = 0; idx < bidRows.Count; idx++)
				Print("Bid Price=" + bidRows[idx].Price + " Volume=" + bidRows[idx].Volume + " Position=" + idx);
		}
		
		// Important to clean up our resources
		protected override void OnTermination()
		{
			// Cleans up our Timer object
			myTimer.Dispose();
		}
        #region Properties
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private SampleCustomEvents[] cacheSampleCustomEvents = null;

        private static SampleCustomEvents checkSampleCustomEvents = new SampleCustomEvents();

        /// <summary>
        /// Reference sample demonstrating how to use custom events
        /// </summary>
        /// <returns></returns>
        public SampleCustomEvents SampleCustomEvents()
        {
            return SampleCustomEvents(Input);
        }

        /// <summary>
        /// Reference sample demonstrating how to use custom events
        /// </summary>
        /// <returns></returns>
        public SampleCustomEvents SampleCustomEvents(Data.IDataSeries input)
        {
            if (cacheSampleCustomEvents != null)
                for (int idx = 0; idx < cacheSampleCustomEvents.Length; idx++)
                    if (cacheSampleCustomEvents[idx].EqualsInput(input))
                        return cacheSampleCustomEvents[idx];

            lock (checkSampleCustomEvents)
            {
                if (cacheSampleCustomEvents != null)
                    for (int idx = 0; idx < cacheSampleCustomEvents.Length; idx++)
                        if (cacheSampleCustomEvents[idx].EqualsInput(input))
                            return cacheSampleCustomEvents[idx];

                SampleCustomEvents indicator = new SampleCustomEvents();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                Indicators.Add(indicator);
                indicator.SetUp();

                SampleCustomEvents[] tmp = new SampleCustomEvents[cacheSampleCustomEvents == null ? 1 : cacheSampleCustomEvents.Length + 1];
                if (cacheSampleCustomEvents != null)
                    cacheSampleCustomEvents.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheSampleCustomEvents = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Reference sample demonstrating how to use custom events
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SampleCustomEvents SampleCustomEvents()
        {
            return _indicator.SampleCustomEvents(Input);
        }

        /// <summary>
        /// Reference sample demonstrating how to use custom events
        /// </summary>
        /// <returns></returns>
        public Indicator.SampleCustomEvents SampleCustomEvents(Data.IDataSeries input)
        {
            return _indicator.SampleCustomEvents(input);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Reference sample demonstrating how to use custom events
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SampleCustomEvents SampleCustomEvents()
        {
            return _indicator.SampleCustomEvents(Input);
        }

        /// <summary>
        /// Reference sample demonstrating how to use custom events
        /// </summary>
        /// <returns></returns>
        public Indicator.SampleCustomEvents SampleCustomEvents(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.SampleCustomEvents(input);
        }
    }
}
#endregion
